@@ -11,12 +11,13 @@ The following interfaces are currently available:
 [1] https://github.com/Paradigm4/shim
 """
 # License: Simplified BSD, 2013
-# Author: Jake Vanderplas <jakevdp@cs.washington.edu>
-# See http://github.com/jakevdp/scidb-py for more information
+# See LICENSE.txt for more information
 
 from __future__ import print_function
-
+import warnings
 import abc
+import os
+import atexit
 
 from ._py3k_compat import urlopen, quote, HTTPError, iteritems, string_type
 
@@ -27,11 +28,23 @@ from .scidbarray import SciDBArray, SciDBDataShape, ArrayAlias, SDB_IND_TYPE
 from .errors import SHIM_ERROR_DICT
 from .utils import broadcastable
 
-__all__ = ['SciDBInterface', 'SciDBShimInterface']
+__all__ = ['SciDBInterface', 'SciDBShimInterface', 'connect']
 
 SCIDB_RAND_MAX = 2147483647  # 2 ** 31 - 1
 
 
+def _df(arr, ind):
+    if not isinstance(arr, ArrayAlias):
+        arr = ArrayAlias(arr)
+    return "{{a.d{i}f}}".format(i=ind).format(a=arr)
+
+
+def _af(arr, ind):
+    if not isinstance(arr, ArrayAlias):
+        arr = ArrayAlias(arr)
+    return "{{a.a{i}f}}".format(i=ind).format(a=arr)
+
+
 def _new_attribute_label(suggestion='val', *arrays):
     """Return a new attribute label
 
@@ -52,6 +65,11 @@ def _new_attribute_label(suggestion='val', *arrays):
 
 
 class SciDBInterface(object):
+
+    def __init__(self):
+        self._created = []
+        atexit.register(self.reap)
+
     """SciDBInterface Abstract Base Class.
 
     This class provides a wrapper to the low-level interface to sciDB.  The
@@ -66,6 +84,12 @@ class SciDBInterface(object):
     """
     __metaclass__ = abc.ABCMeta
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.reap()
+
     @abc.abstractmethod
     def _execute_query(self, query, response=False, n=0, fmt='auto'):
         """Execute a query on the SciDB engine
@@ -128,6 +152,19 @@ class SciDBInterface(object):
         """Get a unique query ID from the database"""
         pass
 
+    def reap(self):
+        """
+        Reap all arrays created via new_array
+        """
+        for array in list(self._created):
+            if (array.datashape is not None) and (not array.persistent):
+                try:
+                    array.reap()
+                except:
+                    pass
+                finally:
+                    self._created.remove(array)
+
     def _db_array_name(self):
         """Return a unique array name for a new array on the database"""
         arr_key = 'py'
@@ -161,7 +198,14 @@ class SciDBInterface(object):
             kwargs['response'] = True
         return self._execute_query("dimensions({0})".format(name), **kwargs)
 
-    def wrap_array(self, scidbname):
+    @property
+    def afl(self):
+        if not hasattr(self, '_afl'):
+            from .afl import AFLNamespace
+            self._afl = AFLNamespace(self)
+        return self._afl
+
+    def wrap_array(self, scidbname, persistent=True):
         """
         Create a new SciDBArray object that references an existing SciDB
         array
@@ -173,11 +217,18 @@ class SciDBInterface(object):
             SciDB array object persistent value will be set to True, and
             the object shape, datashape and data type values will be
             determined by the SciDB array.
+        persistent : boolean
+            If True (default) then array will not be deleted when this
+            variable goes out of scope. Warning: if persistent is set to
+            False, data could be lost!
         """
+        # TODO: use SciDBArray.wrap_array() here; test that it works
         schema = self._show_array(scidbname, fmt='csv')
         datashape = SciDBDataShape.from_schema(schema)
-        return SciDBArray(datashape, self, scidbname, persistent=True)
+        return SciDBArray(datashape, self, scidbname, persistent=persistent)
 
+    # TODO: give the option to pass a user-defined array name
+    #       (use this in copy(), rename(), and others)
     def new_array(self, shape=None, dtype='double', persistent=False,
                   **kwargs):
         """
@@ -214,7 +265,9 @@ class SciDBInterface(object):
             self._execute_query(query)
         else:
             datashape = None
-        return SciDBArray(datashape, self, name, persistent=persistent)
+        result = SciDBArray(datashape, self, name, persistent=persistent)
+        self._created.append(result)
+        return result
 
     def _format_query_string(self, query, *args, **kwargs):
         """Format query string.
@@ -227,6 +280,8 @@ class SciDBInterface(object):
         query = query.format(*args, **kwargs)
         return query
 
+    # TODO: add query_and_store() convenience routine.  Many of the
+    #       uses of query are wrapped in a call to store(...).
     def query(self, query, *args, **kwargs):
         """Perform a query on the database.
 
@@ -346,7 +401,7 @@ class SciDBInterface(object):
             A SciDBArray consisting of all ones.
         """
         arr = self.new_array(shape, dtype, **kwargs)
-        self.query('store(build({0},1),{0})', arr)
+        self.afl.build(arr, 1).eval(out=arr)
         return arr
 
     def zeros(self, shape, dtype='double', **kwargs):
@@ -367,11 +422,10 @@ class SciDBInterface(object):
             A SciDBArray consisting of all zeros.
         """
         schema = SciDBDataShape(shape, dtype, **kwargs).schema
-        arr = self.new_array()
-        self.query('store(build({0}, 0), {1})', schema, arr)
-        return arr
+        return self.afl.build(schema, 0).eval()
 
-    def random(self, shape, dtype='double', lower=0, upper=1, **kwargs):
+    def random(self, shape, dtype='double', lower=0, upper=1, persistent=False,
+               **kwargs):
         """Return an array of random floats between lower and upper
 
         Parameters
@@ -384,6 +438,8 @@ class SciDBInterface(object):
             The lower bound of the random sample (default=0)
         upper : float
             The upper bound of the random sample (default=1)
+        persistent : bool
+            Whether the new array is persistent (default=False)
         **kwargs :
             Additional keyword arguments are passed to SciDBDataShape.
 
@@ -395,15 +451,14 @@ class SciDBInterface(object):
         """
         # TODO: can be done more efficiently
         #       if lower is 0 or upper - lower is 1
+        array = self.new_array(persistent=persistent)
         schema = SciDBDataShape(shape, dtype, **kwargs).schema
-        fill_value = ('random()*{0}/{2}+{1}'.format(upper - lower, lower,
-                                                    float(SCIDB_RAND_MAX)))
-        arr = self.new_array()
-        self.query('store(build({0}, {1}), {2})', schema, fill_value, arr)
-        return arr
+        rng = (upper - lower) / float(SCIDB_RAND_MAX)
+        fill_value = 'random()*{0}+{1}'.format(rng, lower)
+        return self.afl.build(schema, fill_value).eval(out=array)
 
     def randint(self, shape, dtype='uint32', lower=0, upper=SCIDB_RAND_MAX,
-                **kwargs):
+                persistent=False, **kwargs):
         """Return an array of random integers between lower and upper
 
         Parameters
@@ -416,6 +471,8 @@ class SciDBInterface(object):
             The lower bound of the random sample (default=0)
         upper : float
             The upper bound of the random sample (default=2147483647)
+        persistent : bool
+            Whether the array is persistent (default=False)
         **kwargs :
             Additional keyword arguments are passed to SciDBDataShape.
 
@@ -425,11 +482,10 @@ class SciDBInterface(object):
             A SciDBArray consisting of random integers, uniformly distributed
             between `lower` and `upper`.
         """
+        array = self.new_array(persistent=persistent)
         schema = SciDBDataShape(shape, dtype, **kwargs).schema
         fill_value = 'random() % {0} + {1}'.format(upper - lower, lower)
-        arr = self.new_array()
-        self.query('store(build({0}, {1}), {2})', schema, fill_value, arr)
-        return arr
+        return self.afl.build(schema, fill_value).eval(out=array)
 
     def arange(self, start, stop=None, step=1, dtype=None, **kwargs):
         """arange([start,] stop[, step,], dtype=None, **kwargs)
@@ -486,8 +542,10 @@ class SciDBInterface(object):
         size = int(np.ceil((stop - start) * 1. / step))
 
         arr = self.new_array(size, dtype, **kwargs)
-        self.query("store(build({A}, {start} + {step} * {A.d0}), {A})",
-                   A=arr, start=start, step=step)
+
+        fill_value = '{0} + {1} * {2}'.format(start, step,
+                                              arr.dim_names[0])
+        self.afl.build(arr, fill_value).eval(out=arr)
         return arr
 
     def linspace(self, start, stop, num=50,
@@ -537,8 +595,8 @@ class SciDBInterface(object):
             step = (stop - start) * 1. / num
 
         arr = self.new_array(num, **kwargs)
-        self.query("store(build({A}, {start} + {step} * {A.d0}), {A})",
-                   A=arr, start=start, step=step)
+        fill_value = '{0} + {1} * {2}'.format(start, step, arr.dim_names[0])
+        self.afl.build(arr, fill_value).eval(out=arr)
 
         if retstep:
             return arr, step
@@ -565,11 +623,13 @@ class SciDBInterface(object):
             A SciDBArray containint an [n x n] identity matrix
         """
         arr = self.new_array((n, n), dtype, **kwargs)
+        d0, d1 = arr.dim_names
+        diag = "{0}={1}".format(d0, d1)
         if sparse:
-            query = 'store(build_sparse({A},1,{A.d0}={A.d1}),{A})'
+            query = self.afl.build_sparse(arr, 1, diag)
         else:
-            query = 'store(build({A},iif({A.d0}={A.d1},1,0)),{A})'
-        self.query(query, A=arr)
+            query = self.afl.build(arr, 'iif(%s,1,0)' % diag)
+        query.eval(out=arr)
         return arr
 
     def dot(self, A, B):
@@ -621,8 +681,11 @@ class SciDBInterface(object):
         if B.sdbtype.nullable[0]:
             B = B.substitute(0)
 
-        C = self.new_array()
-        self.query('store(multiply({0},{1}),{2})', A, B, C)
+        # TODO: is there a more efficient way to do this than instantiating
+        #       an array of zeros?
+        # TODO: use spgemm() when the matrices are sparse
+        C = self.zeros((A.shape[0], B.shape[1]), dtype=A.dtype)
+        self.afl.gemm(A, B, C).eval(out=C)
 
         if C.shape == output_shape:
             return C
@@ -653,7 +716,7 @@ class SciDBInterface(object):
         """
         if (A.ndim != 2):
             raise ValueError("svd requires 2-dimensional arrays")
-        self.query("load_library('dense_linear_algebra')")
+        self.afl.load_library("'dense_linear_algebra'")
 
         out_dict = dict(U=return_U, S=return_S, VT=return_VT)
 
@@ -661,8 +724,7 @@ class SciDBInterface(object):
         ret = []
         for output in ['U', 'S', 'VT']:
             if out_dict[output]:
-                ret.append(self.new_array())
-                self.query("store(gesvd({0}, '{1}'),{2})", A, output, ret[-1])
+                ret.append(self.afl.gesvd(A, "'%s'" % output).eval())
         return tuple(ret)
 
     def from_array(self, A, instance_id=0, **kwargs):
@@ -677,19 +739,19 @@ class SciDBInterface(object):
             (default=0; see SciDB documentation)
         **kwargs :
             Additional keyword arguments are passed to new_array()
-        
+
         Returns
         -------
         arr : SciDBArray
             SciDB Array object built from the input array
         """
+        q = self.afl.quote
         A = np.asarray(A)
         instance_id = int(instance_id)
-        filename = self._upload_bytes(A.tostring(order='C'))
+        filename = q(self._upload_bytes(A.tostring(order='C')))
         arr = self.new_array(A.shape, A.dtype, **kwargs)
-        self.query("load({0},'{1}',{2},'{3}')",
-                   arr, filename, instance_id,
-                   arr.sdbtype.bytes_fmt)
+        self.afl.load(arr, filename, instance_id,
+                      q(arr.sdbtype.bytes_fmt)).eval(store=False)
         return arr
 
     def from_dataframe(self, A, instance_id=0, **kwargs):
@@ -704,7 +766,7 @@ class SciDBInterface(object):
             (default=0; see SciDB documentation)
         **kwargs :
             Additional keyword arguments are passed to new_array()
-        
+
         Returns
         -------
         arr : SciDBArray
@@ -723,7 +785,7 @@ class SciDBInterface(object):
         arr = self.new_array(shape=A_rec.shape,
                              dtype=A_rec.dtype.descr[1:],
                              **kwargs)
-        self.query("redimension_store({0},{1})", A_sdb, arr)
+        self.afl.redimension_store(A_sdb, arr).eval(store=False)
         return arr
 
     def from_sparse(self, A, instance_id=0, **kwargs):
@@ -739,7 +801,7 @@ class SciDBInterface(object):
             (default=0; see SciDB documentation)
         **kwargs :
             Additional keyword arguments are passed to new_array()
-        
+
         Returns
         -------
         arr : SciDBArray
@@ -754,11 +816,9 @@ class SciDBInterface(object):
         if 'dim_names' not in kwargs:
             kwargs['dim_names'] = ['i0', 'i1']
 
-        attr_name = 'f0'
-
         if len(kwargs['dim_names']) != 2:
             raise ValueError("dim_names must have two dimensions")
-        d1, d2 = kwargs['dim_names']    
+        d1, d2 = kwargs['dim_names']
 
         # first flatten the array & indices
         # We'll treat the general case where A.data can be a record array,
@@ -777,7 +837,7 @@ class SciDBInterface(object):
 
         # redimension the flat array to a sparse array
         arr = self.new_array(A.shape, A.dtype, **kwargs)
-        self.query("redimension_store({0},{1})", arr_flat, arr)
+        self.afl.redimension_store(arr_flat, arr).eval(store=False)
         return arr
 
     def toarray(self, A, transfer_bytes=True):
@@ -792,7 +852,7 @@ class SciDBInterface(object):
         """Convert a SciDB array to a pandas dataframe"""
         return A.todataframe(transfer_bytes=transfer_bytes)
 
-    def from_file(self, filename, **kwargs):
+    def _from_file(self, filename, **kwargs):
         # TODO: allow creation of arrays from uploaded files
         # TODO: allow creation of arrays from pre-existing files within the
         #       database
@@ -801,11 +861,12 @@ class SciDBInterface(object):
     def _apply_func(self, A, func):
         # TODO: new value name could conflict.  How to generate a unique one?
         # TODO: add optional ``out`` argument as in numpy
-        arr = self.new_array()
-        self.query("store(project(apply({A},{func}_{A.a0},{func}({A.a0})),"
-                   "{func}_{A.a0}), {arr})",
-                   A=A, func=func, arr=arr)
-        return arr
+        f = self.afl
+        att = A.att(0)
+        newatt = "{0}_{1}".format(func, att)
+        expr = "{0}({1})".format(func, att)
+        q = f.papply(A, newatt, expr)
+        return q.eval()
 
     def sin(self, A):
         """Element-wise trigonometric sine"""
@@ -844,56 +905,94 @@ class SciDBInterface(object):
         return self._apply_func(A, 'log10')
 
     def min(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis minimum.
+
+        see :meth:`SciDBArray.min` """
         return A.min(index, scidb_syntax)
 
     def max(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis maximum.
+
+        see :meth:`SciDBArray.max` """
         return A.max(index, scidb_syntax)
 
     def sum(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis sum.
+
+        see :meth:`SciDBArray.sum` """
         return A.sum(index, scidb_syntax)
 
     def var(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis variance.
+
+        see :meth:`SciDBArray.var` """
         return A.var(index, scidb_syntax)
 
     def stdev(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis standard deviation.
+
+        see :meth:`SciDBArray.stdev` """
         return A.stdev(index, scidb_syntax)
 
     def std(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis standard deviation.
+
+        see :meth:`SciDBArray.std` """
         return A.std(index, scidb_syntax)
 
     def avg(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis average.
+
+        see :meth:`SciDBArray.avg` """
         return A.avg(index, scidb_syntax)
 
     def mean(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis mean.
+
+        see :meth:`SciDBArray.mean` """
         return A.mean(index, scidb_syntax)
 
     def count(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis count.
+
+        see :meth:`SciDBArray.count` """
         return A.count(index, scidb_syntax)
 
     def approxdc(self, A, index=None, scidb_syntax=False):
+        """
+        Array or axis unique element estimate.
+
+        see :meth:`SciDBArray.approxdc` """
         return A.approxdc(index, scidb_syntax)
 
     def substitute(self, A, value):
+        """
+        Replace null values in an array
+
+        See :meth:`SciDBArray.substitute`
+        """
         return A.substitute(value)
 
     def merge(self, A, B):
         """Merge two arrays"""
         # TODO: pre-check for non-matching arrays?
-        arr = self.new_array()
-        self.query("store(merge({0},{1}),{2})", A, B, arr)
-        return arr
+        return self.afl.merge(A, B).eval()
 
     def join(self, *args):
         """
         Perform a series of array joins on the arguments
         and return the result.
         """
-        last = args[0]
-        for arg in args[1:]:
-            arr = self.new_array()
-            self.query('store(join({0},{1}), {2})', last, arg, arr)
-            last = arr
-        return last
+        return reduce(self.afl.join, args).eval()
 
     def cross_join(self, A, B, *dims):
         """Perform a cross-join on arrays A and B.
@@ -905,42 +1004,39 @@ class SciDBInterface(object):
             The remaining arguments are tuples of dimension indices which
             should be joined.
         """
-        dims = ','.join(" {{A.d{0}f}}, {{B.d{1}f}}".format(*tup)
-                        for tup in dims)
-        if dims:
-            dims = ',' + dims
-        query = ('store(cross_join({A}, {B}'
-                 + dims
-                 + '), {arr})')
-        arr = self.new_array()
-        self.query(query, A=A, B=B, arr=arr)
-        return arr
+        dims = [_df(arr, index)
+                for dim in dims
+                for arr, index in zip([A, B], dim)]
+        return self.afl.cross_join(A, B, *dims).eval()
 
     def _join_operation(self, left, right, op):
         """Perform a join operation across arrays or values.
 
         See e.g. SciDBArray.__add__ for an example usage.
         """
+        f = self.afl
+
         if isinstance(left, SciDBArray):
             left_name = left.name
-            left_fmt = '{left.a0f}'
+            left_fmt = _af(left, 0)
             left_is_sdb = True
         else:
             left_name = None
-            left_fmt = '{left}'
+            left_fmt = left
             left_is_sdb = False
 
         if isinstance(right, SciDBArray):
             right_name = right.name
-            right_fmt = '{right.a0f}'
+            right_fmt = _af(right, 0)
             right_is_sdb = True
         else:
             right_name = None
-            right_fmt = '{right}'
+            right_fmt = right
             right_is_sdb = False
 
         # some common names needed below
-        op = op.format(left=left_fmt, right=right_fmt)
+        op = op(left_fmt, right_fmt)
+
         aL = aR = None
         permutation = None
 
@@ -959,11 +1055,10 @@ class SciDBInterface(object):
                 attr = _new_attribute_label('x', left, right)
                 if left_name == right_name:
                     # same array: we can do this without a join
-                    query = ("store(project(apply({left}, {attr}, "
-                             + op + "), {attr}), {arr})")
+                    return f.papply(left, attr, op).eval()
                 else:
-                    query = ("store(project(apply(join({left},{right}),{attr},"
-                             + op + "), {attr}), {arr})")
+                    q = f.papply(f.join(left, right), attr, op)
+                    return q.eval()
 
             # array shapes are broadcastable: use a cross_join
             elif broadcastable(left.shape, right.shape):
@@ -990,34 +1085,33 @@ class SciDBInterface(object):
 
                 # build the left slice query if needed
                 if left_slices:
-                    dims = ','.join("{{left.d{0}f}},0".format(sl)
-                                    for sl in left_slices)
-                    left_query = "slice({left}," + dims + ") as {aL}"
                     aL = ArrayAlias(left, "alias_left")
+                    dims = [item for sl in left_slices
+                            for item in [_df(left, sl), 0]]
+                    left_query = f.as_(f.slice(left, *dims), aL).query
                 else:
-                    left_query = "{aL}"
+                    left_query = left
                     aL = left
 
                 # build the right slice query if needed
                 if right_slices:
-                    dims = ','.join("{{right.d{0}f}},0".format(sl)
-                                    for sl in right_slices)
-                    right_query = "slice({right}," + dims + ") as {aR}"
                     aR = ArrayAlias(right, "alias_right")
+                    dims = [item for sl in right_slices
+                            for item in [_df(right, sl), 0]]
+                    right_query = f.as_(f.slice(right, *dims), aR).query
                 else:
-                    right_query = "{aR}"
+                    right_query = right
                     aR = right
 
                 # build the cross_join query
-                dims = ','.join("{{aL.d{0}f}}, {{aR.d{1}f}}".format(i, j)
-                                for i, j in join_indices)
-                if dims:
-                    dims = ',' + dims
-
-                query = ("store(project(apply(cross_join(" +
-                         left_query + "," + right_query + dims + "),{attr}," +
-                         op + "), {attr}), {arr})")
+                dims = [_df(arr, att)
+                        for atts in join_indices
+                        for arr, att in zip([aL, aR], atts)]
                 attr = _new_attribute_label('x', left, right)
+                query = f.papply(f.cross_join(
+                                 left_query, right_query, *dims),
+                                 attr, op)
+                result = query.eval()
 
                 # determine the dimension permutation
                 # Here's the problem: cross_join puts all the left array dims
@@ -1055,6 +1149,10 @@ class SciDBInterface(object):
                 if permutation == range(len(permutation)):
                     permutation = None
 
+                if permutation is not None:
+                    result = result.transpose(permutation)
+                return result
+
             else:
                 raise ValueError("Array of shape {0} can not be "
                                  "broadcasted with array of shape "
@@ -1063,27 +1161,20 @@ class SciDBInterface(object):
         # only left entry is a SciDBArray
         elif left_is_sdb:
             try:
-                _ = float(right)
+                float(right)
             except:
                 raise ValueError("rhs must be a scalar or SciDBArray")
             attr = _new_attribute_label('x', left)
-            query = ("store(project(apply({left}, {attr}, "
-                     + op + "), {attr}), {arr})")
+            return f.papply(left, attr, op).eval()
 
         # only right entry is a SciDBArray
         elif right_is_sdb:
             try:
-                _ = float(left)
+                float(left)
             except:
                 raise ValueError("lhs must be a scalar or SciDBArray")
             attr = _new_attribute_label('x', right)
-            query = ("store(project(apply({right}, {attr}, "
-                     + op + "), {attr}), {arr})")
-
-        arr = self.new_array()
-        self.query(query, left=left, right=right,
-                   aL=aL, aR=aR,
-                   attr=attr, arr=arr)
+            return f.papply(right, attr, op).eval()
 
         # reorder the dimensions if needed (for cross_join)
         if permutation is not None:
@@ -1092,6 +1183,7 @@ class SciDBInterface(object):
 
 
 class SciDBShimInterface(SciDBInterface):
+
     """HTTP interface to SciDB via shim [1]_
 
     Parameters
@@ -1101,12 +1193,15 @@ class SciDBShimInterface(SciDBInterface):
 
     [1] https://github.com/Paradigm4/shim
     """
+
     def __init__(self, hostname):
+        super(SciDBShimInterface, self).__init__()
         self.hostname = hostname.rstrip('/')
         try:
-            urlopen(self.hostname)
-        except HTTPError:
-            raise ValueError("Invalid hostname: {0}".format(self.hostname))
+            self._get_uid()
+        except:
+            raise ValueError("Could not connect to a SciDB instance at {0}"
+                             .format(self.hostname))
 
     def _get_uid(self):
         # load a library to get a query id
@@ -1163,7 +1258,7 @@ class SciDBShimInterface(SciDBInterface):
 
     def _shim_release_session(self, session_id):
         url = self._shim_url('release_session', id=session_id)
-        result = self._shim_urlopen(url)
+        self._shim_urlopen(url)
 
     def _shim_execute_query(self, session_id, query, save=None, release=False):
         url = self._shim_url('execute_query',
@@ -1184,7 +1279,7 @@ class SciDBShimInterface(SciDBInterface):
 
     def _shim_cancel(self, session_id):
         url = self._shim_url('cancel', id=session_id)
-        result = self._shim_urlopen(url)
+        self._shim_urlopen(url)
 
     def _shim_read_lines(self, session_id, n):
         url = self._shim_url('read_lines', id=session_id, n=n)
@@ -1208,3 +1303,18 @@ class SciDBShimInterface(SciDBInterface):
         result = requests.post(url, files=dict(fileupload=data))
         scidb_filename = result.text.strip()
         return scidb_filename
+
+
+def connect(url=None):
+    """
+    Connect to a SciDB instance
+
+    Parameters
+    ----------
+    url : str (optional)
+        Connection URL. If not provided, will fall back to
+        the SCIDB_URL environment variable (if present),
+        or http://127.0.0.1:8080
+    """
+    url = url or os.environ.get('SCIDB_URL', 'http://127.0.0.1:8080')
+    return SciDBShimInterface(url)
