@@ -1,8 +1,7 @@
 """SciDB Array Wrapper"""
 
-# License: Simplified BSD, 2013
-# Author: Jake Vanderplas <jakevdp@cs.washington.edu>
-# See http://github.com/jakevdp/scidb-py for more information
+# License: Simplified BSD, 2014
+# See LICENSE.txt for more information
 
 from __future__ import print_function
 
@@ -11,7 +10,7 @@ import re
 
 import warnings
 from copy import copy
-from .errors import SciDBError
+from .errors import SciDBError, SciDBForbidden
 
 # Numpy 1.7 meshgrid backport
 from .utils import meshgrid
@@ -42,6 +41,7 @@ SDB_IND_TYPE = 'int64'
 
 
 class sdbtype(object):
+
     """SciDB data type class.
 
     This class encapsulates the information about the datatype of SciDB
@@ -52,6 +52,7 @@ class sdbtype(object):
     typecode : string, list, sdbtype, or dtype
         An object representing a datatype.
     """
+
     def __init__(self, typecode):
         if isinstance(typecode, sdbtype):
             self.schema = typecode.schema
@@ -179,7 +180,9 @@ class sdbtype(object):
 
 
 class SciDBDataShape(object):
+
     """Object to store SciDBArray data type and shape"""
+
     def __init__(self, shape, typecode, dim_names=None,
                  chunk_size=1000, chunk_overlap=0):
         # Process array shape
@@ -202,7 +205,7 @@ class SciDBDataShape(object):
             vals = (int(m.groups()[0]) for m in matches)
             try:
                 start = max(vals) + 1
-            except ValueError: # empty sequence
+            except ValueError:  # empty sequence
                 start = 0
             dim_names = ['i{0}'.format(i)
                          for i in range(start, start + len(self.shape))]
@@ -295,9 +298,11 @@ class SciDBDataShape(object):
 
 
 class ArrayAlias(object):
+
     """
     An alias object used for constructing queries
     """
+
     def __init__(self, arr, name=None):
         self.arr = arr
         if name is None:
@@ -346,29 +351,111 @@ class ArrayAlias(object):
 
 
 class SciDBArray(object):
+
     """SciDBArray class
 
     It is not recommended to instantiate this class directly; use a
     convenience routine from SciDBInterface.
     """
+
     def __init__(self, datashape, interface, name, persistent=False):
         self._datashape = datashape
         self.interface = interface
         self.name = name
         self.persistent = persistent
 
+    @property
+    def afl(self):
+        """
+        An alias to the AFL namespace
+        """
+        return self.interface.afl
+
+    def rename(self, new_name, persistent=False):
+        """Rename the array in the database, optionally making the new
+        array persistent.
+
+        Parameters
+        ----------
+        new_name : string
+            must be a valid array name which does not already
+            exist in the database.
+        persistent : boolean (optional)
+            specify whether the new array is persistent (default=False)
+
+        Returns
+        -------
+        self : SciDBArray
+            return a pointer to self
+        """
+        new_name = str(new_name)
+
+        if new_name in self.interface.list_arrays():
+            raise ValueError("Cannot use name {0}. "
+                             "An array with that name "
+                             "already exists.".format(new_name))
+        self.afl.rename(self, new_name).eval(store=False)
+        self.name = new_name
+        self.persistent = persistent
+        return self
+
+    def copy(self, new_name=None, persistent=False):
+        """Make a copy of the array in the database
+
+        Parameters
+        ----------
+        new_name : string (optional)
+            if specifiedmust be a valid array name which does not already
+            exist in the database.
+        persistent : boolean (optional)
+            specify whether the new array is persistent (default=False)
+
+        Returns
+        -------
+        copy : SciDBArray
+            return a copy of the original array
+        """
+        # TODO: allow new_name to be specified in interface.new_array()
+        if new_name is not None:
+            new_name = str(new_name)
+            if new_name in self.interface.list_arrays():
+                raise ValueError("Cannot use name {0}. "
+                                 "An array with that name "
+                                 "already exists.".format(new_name))
+
+        arr = self.interface.new_array(persistent=persistent)
+        if new_name is not None:
+            arr.name = new_name
+
+        self.afl.store(self, arr).eval(store=False)
+        return arr
+
     def alias(self, name=None):
         """Return an alias of the array, optionally with a new name"""
         return ArrayAlias(self, name)
 
     def dimension(self, d):
-        """Return the dimension name of the array"""
+        """Return the dimension name of the array
+
+        Parameters
+        ----------
+        d : int
+           The index of the dimension to lookup
+        """
         return self.datashape.dim_names[d]
 
     def attribute(self, a):
-        """Return the attribute name of the array"""
+        """Return the attribute name of the array.
+
+        Parameters
+        ----------
+        a : int
+           Index of the attribute to lookup
+        """
         return self.datashape.sdbtype.full_rep[a][0]
 
+    att = attribute
+
     @property
     def datashape(self):
         if self._datashape is None:
@@ -411,15 +498,38 @@ class SciDBArray(object):
     def dtype(self):
         return self.datashape.dtype
 
-    def __del__(self):
-        if (self.datashape is not None) and (not self.persistent):
-            self.interface.query("remove({0})", self)
+    def reap(self, ignore=False):
+        """
+        Delete this object from the database if it isn't persistent.
+
+        Parameters
+        ----------
+        ignore : bool (default False)
+            If False and the array is persistent, then reap raises an error
+            If True and the array is persistent, reap does nothing
+
+        Raises
+        ------
+        SciDBForbidden if ``persistent=True`` and ``ignore=False`
+        """
+        if self.persistent:
+            if not ignore:
+                raise SciDBForbidden("Cannot reap: persistent=True")
+            return
+
+        if (self.datashape is not None):
+            self.interface.query("remove({0})", self.name)
+            self.name = '__DELETED__'
+            self.interface = None
 
     def __repr__(self):
         show = self.interface._show_array(self.name, fmt='csv').split('\n')
         return "SciDBArray({0})".format(show[1])
 
     def contents(self, **kwargs):
+        """
+        Return a string representation of the array contents
+        """
         return repr(self) + '\n' + self.interface._scan_array(self.name,
                                                               **kwargs)
 
@@ -434,9 +544,8 @@ class SciDBArray(object):
         --------
         nonnull()
         """
-        query = "count({0})".format(self.name)
-        response = self.interface._execute_query(query, response=True,
-                                                 fmt='(int64)')
+        query = self.afl.count(self)
+        response = query.eval(response=True, fmt='(int64)', store=False)
         return np.fromstring(response, dtype='int64')[0]
 
     def nonnull(self, attr=0):
@@ -469,9 +578,8 @@ class SciDBArray(object):
 
         for i in range(attr.size):
             attr_name = self.attribute(attr.flat[i])
-            query = "aggregate({0}, count({1}))".format(self.name, attr_name)
-            response = self.interface._execute_query(query, response=True,
-                                                     fmt='(int64)')
+            query = self.afl.aggregate(self, 'count(%s)' % attr_name)
+            response = query.eval(response=True, store=False, fmt='(int64)')
             nonnull.flat[i] = np.fromstring(response, dtype='int64')[0]
         return nonnull
 
@@ -535,7 +643,7 @@ class SciDBArray(object):
             # This is needed for both sparse and dense outputs
             bytes_rep = self.interface._scan_array(self.name, n=0,
                                                    fmt=self.sdbtype.bytes_fmt)
-            bytes_arr = np.fromstring(bytes_rep, dtype=dtype)
+            bytes_arr = np.atleast_1d(np.fromstring(bytes_rep, dtype=dtype))
 
         if array_is_sparse:
             # perform a CSV query to find all non-empty index tuples.
@@ -547,8 +655,9 @@ class SciDBArray(object):
                 raise ValueError("Fatal: unexpected array labels.")
 
             # convert the ASCII representation into a numpy record array
-            arr = genfromstr(str_rep, delimiter=',', skip_header=1,
-                             dtype=full_dtype)
+            arr = np.atleast_1d(genfromstr(str_rep, delimiter=',',
+                                           skip_header=1,
+                                           dtype=full_dtype))
 
             if transfer_bytes:
                 # replace parsed ASCII columns with more accurate bytes
@@ -719,23 +828,17 @@ class SciDBArray(object):
 
         # special case: accessing a single element (no slices)
         if all(not isinstance(i, slice) for i in indices):
-            limits = map(int, indices + indices)
-            arr = self.interface.new_array()
-            self.interface.query("store(subarray({0},{2}),{1})",
-                                 self, arr, ','.join(str(L) for L in limits))
-            return arr.toarray().flat[0]
+            limits = list(map(int, indices + indices))
+            q = self.afl.subarray(self, *limits)
+            return q.toarray().flat[0]
 
         # if any of the slices are integers, we'll first use SciDB's
         # slice() operation on these
-        slices = [(dim, s) for (dim, s) in enumerate(indices)
-                  if not isinstance(s, slice)]
+        slices = [item for (d, s) in zip(self.dim_names, indices)
+                  if not isinstance(s, slice)
+                  for item in (d, s)]
         if slices:
-            query = ("store(slice({X}," +
-                     ','.join('{{X.d{0}}}, {1}'.format(*slc)
-                              for slc in slices) +
-                     "), {arr})")
-            arr1 = self.interface.new_array()
-            self.interface.query(query, X=self, arr=arr1)
+            arr1 = self.afl.slice(self, *slices).eval()
         else:
             arr1 = self
 
@@ -747,18 +850,14 @@ class SciDBArray(object):
         # if a subarray is required, then call the subarray() command
         if any(i[0] != 0 or i[1] != s for (i, s) in zip(indices, shape)):
             limits = [i[0] for i in indices] + [i[1] - 1 for i in indices]
-            arr2 = self.interface.new_array()
-            self.interface.query("store(subarray({0},{2}),{1})",
-                                 arr1, arr2, ','.join(str(L) for L in limits))
+            arr2 = self.afl.subarray(arr1, *limits).eval()
         else:
             arr2 = arr1
 
         # if thinning is required, then call the thin() command
         if any(i[2] != 1 for i in indices):
             steps = sum([[0, i[2]] for i in indices], [])
-            arr3 = self.interface.new_array()
-            self.interface.query("store(thin({0},{2}),{1})",
-                                 arr2, arr3, ','.join(str(st) for st in steps))
+            arr3 = self.afl.thin(arr2, *steps).eval()
         else:
             arr3 = arr2
 
@@ -767,48 +866,48 @@ class SciDBArray(object):
     # join operations: note that these ignore all but the first attribute
     # of each array.
     def __add__(self, other):
-        return self.interface._join_operation(self, other, '{left}+{right}')
+        return self.interface._join_operation(self, other, self.afl.add)
 
     def __radd__(self, other):
-        return self.interface._join_operation(other, self, '{left}+{right}')
+        return self.interface._join_operation(other, self, self.afl.add)
 
     def __sub__(self, other):
-        return self.interface._join_operation(self, other, '{left}-{right}')
+        return self.interface._join_operation(self, other, self.afl.sub)
 
     def __rsub__(self, other):
-        return self.interface._join_operation(other, self, '{left}-{right}')
+        return self.interface._join_operation(other, self, self.afl.sub)
 
     def __mul__(self, other):
-        return self.interface._join_operation(self, other, '{left}*{right}')
+        return self.interface._join_operation(self, other, self.afl.mul)
 
     def __rmul__(self, other):
-        return self.interface._join_operation(other, self, '{left}*{right}')
+        return self.interface._join_operation(other, self, self.afl.mul)
 
     def __div__(self, other):
-        return self.interface._join_operation(self, other, '{left}/{right}')
+        return self.interface._join_operation(self, other, self.afl.div)
 
     def __truediv__(self, other):
-        return self.interface._join_operation(self, other, '{left}/{right}')
+        return self.interface._join_operation(self, other, self.afl.div)
 
     def __rdiv__(self, other):
-        return self.interface._join_operation(other, self, '{left}/{right}')
+        return self.interface._join_operation(other, self, self.afl.div)
 
     def __rtruediv__(self, other):
-        return self.interface._join_operation(other, self, '{left}/{right}')
+        return self.interface._join_operation(other, self, self.afl.div)
 
     def __mod__(self, other):
-        return self.interface._join_operation(self, other, '{left}%{right}')
+        return self.interface._join_operation(self, other, self.afl.mod)
 
     def __rmod__(self, other):
-        return self.interface._join_operation(other, self, '{left}%{right}')
+        return self.interface._join_operation(other, self, self.afl.mod)
 
     def __pow__(self, other):
         return self.interface._join_operation(self, other,
-                                              'pow({left},{right})')
+                                              self.afl.pow)
 
     def __rpow__(self, other):
         return self.interface._join_operation(other, self,
-                                              'pow({left},{right})')
+                                              self.afl.pow)
 
     def __abs__(self):
         return self.interface._apply_func(self, 'abs')
@@ -845,8 +944,7 @@ class SciDBArray(object):
                     pass
 
         if not axes:
-            arr = self.interface.new_array()
-            self.interface.query("store(transpose({0}), {1})", self, arr)
+            arr = self.afl.transpose(self).eval()
         else:
             # first validate the axes
             axes = [(a + self.ndim if a < 0 else a) for a in axes]
@@ -865,8 +963,7 @@ class SciDBArray(object):
                                            chunk_size=chunk_size,
                                            chunk_overlap=chunk_overlap,
                                            dim_names=dim_names)
-            self.interface.query("redimension_store({input}, {output})",
-                                 input=self, output=arr)
+            self.afl.redimension_store(self, arr).eval(store=False)
         return arr
 
     # This allows the transpose of A to be computed via A.T
@@ -893,9 +990,7 @@ class SciDBArray(object):
         arr = self.interface.new_array(shape=shape,
                                        dtype=self.sdbtype,
                                        **kwargs)
-        self.interface.query("store(reshape({input}, {output}), {output})",
-                             input=self, output=arr)
-        return arr
+        return self.afl.reshape(self, arr).eval(out=arr)
 
     def substitute(self, value):
         """Reshape data into a new array, substituting a default for any nulls.
@@ -909,11 +1004,9 @@ class SciDBArray(object):
         arr : SciDBArray
             new non-nullable array
         """
-        qstring = "store(substitute({A}, build({T}[i=0:0,1,0],{value})),{arr})"
-        arr = self.interface.new_array()
-        self.interface.query(qstring, A=self, arr=arr,
-                             T=self.sdbtype, value=value)
-        return arr
+        b = self.afl.build('%s[i=0:0,1,0]' % self.sdbtype, value)
+        q = self.afl.substitute(self, b)
+        return q.eval()
 
     def _aggregate_operation(self, agg, index=None, scidb_syntax=True):
         """Perform an aggregation query
@@ -947,12 +1040,8 @@ class SciDBArray(object):
         to Python users, but keep a flag which allows SciDB-like behavior.
         """
         # TODO: add optional ``out`` argument, as in numpy
-
-        if index is None:
-            # for both SciDB-style and numpy-style, with no index specified
-            # we aggregate over the entire array
-            idx_string = ''
-        else:
+        idx_args = []
+        if index is not None:
             try:
                 ind = tuple(index)
             except:
@@ -975,47 +1064,224 @@ class SciDBArray(object):
                 ind = tuple(i for i in range(self.ndim) if i not in ind)
 
             # corner case where indices are an empty tuple
-            if len(ind) == 0:
-                idx_string = ''
-            else:
-                idx_string = ',' + ', '.join(['{{A.d{0}}}'.format(i)
-                                              for i in ind])
+            if len(ind) > 0:
+                idx_args = [self.dim_names[i] for i in ind]
 
-        qstring = ("store(aggregate({A}, {agg}({A.a0})" + idx_string
-                   + "), {arr})")
-
-        arr = self.interface.new_array()
-        self.interface.query(qstring, A=self, arr=arr, agg=agg)
-        return arr
+        agg = "{agg}({att})".format(agg=agg, att=self.att(0))
+        q = self.afl.aggregate(self, agg, *idx_args)
+        return q.eval()
 
     def min(self, index=None, scidb_syntax=False):
+        """
+        Return the minimum of the array or the minimum along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('min', index, scidb_syntax)
 
     def max(self, index=None, scidb_syntax=False):
+        """
+        Return the maximum of the array or the maximum along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('max', index, scidb_syntax)
 
     def sum(self, index=None, scidb_syntax=False):
+        """
+        Return the sum of the array or the sum along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('sum', index, scidb_syntax)
 
     def var(self, index=None, scidb_syntax=False):
+        """
+        Return the variance of the array or the variance along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('var', index, scidb_syntax)
 
     def stdev(self, index=None, scidb_syntax=False):
+        """
+        Return the standard deviation of the array or along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('stdev', index, scidb_syntax)
 
     def std(self, index=None, scidb_syntax=False):
+        """
+        Return the standard deviation of the array or along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+
+        Notes
+        -----
+        Identical to :meth:`SciDBArray.stdev`
+
+        """
         return self._aggregate_operation('stdev', index, scidb_syntax)
 
     def avg(self, index=None, scidb_syntax=False):
+        """
+        Return the average of the array or the average along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('avg', index, scidb_syntax)
 
     def mean(self, index=None, scidb_syntax=False):
+        """
+        Return the average of the array or the average along an axis.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+
+        Notes
+        -----
+        Identical to :meth:`SciDBArray.avg`
+        """
         return self._aggregate_operation('avg', index, scidb_syntax)
 
     def count(self, index=None, scidb_syntax=False):
+        """
+        Return the count of the array or the count along an axis.
+
+        The count is equal to the number of nonnull elements.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('count', index, scidb_syntax)
 
     def approxdc(self, index=None, scidb_syntax=False):
+        """
+        Return the number of distinct values of the array or along an axis.
+
+        The distinct count is an estimate only.
+
+        Parameters
+        ----------
+        index : int, optional
+            Axis along which to operate. By default, flattened input is used.
+        scidb_syntax : bool, optional (default=False)
+            If False, index follows the numpy convention
+            (i.e., the array is collapsed over the index'th axis).
+            If True, index follows the SciDB convention
+            (i.e., the array is collapsed over all axes *except* index)
+
+        Returns
+        -------
+        A SciDB array
+        """
         return self._aggregate_operation('approxdc', index, scidb_syntax)
 
     def regrid(self, size, aggregate="avg"):
@@ -1049,9 +1315,7 @@ class SciDBArray(object):
             raise ValueError("aggregate='{0}' "
                              "not recognized".format(aggregate))
 
-        arr = self.interface.new_array()
-        query = "store(regrid({A}, {dims}, {agg}({A.a0})), {arr})"
-        self.interface.query(query,
-                             A=self, agg=aggregate, arr=arr,
-                             dims=','.join(map(str, sizes)))
-        return arr
+        agg = "{agg}({att})".format(agg=aggregate, att=self.att(0))
+        args = list(map(str, sizes)) + [agg]
+        q = self.afl.regrid(self, *args)
+        return q.eval()
